<?php
require_once 'abstract.php';

/**
 *
 */
class Mage_Shell_Log extends Mage_Shell_Abstract
{

    const LOG_NAME = 'sf-remove.log';

    /**
     * @var bool
     */
    protected $files = false;

    /**
     * @var bool
     */
    protected $dbChanges = false;

    /**
     * @var bool
     */
    protected $showQueries = false;

    /**
     * @var bool
     */
    protected $_success = true;

    /**
     * @var array
     */
    protected $_moduleFiles = array(
        'app/code/community/TNW/Quote',
        'app/design/adminhtml/default/default/layout/tnw_quote.xml',
        'app/design/adminhtml/default/default/template/tnw_quote',
        'app/etc/modules/TNW_Quote.xml',

        'app/code/community/TNW/Accountingseed',
        'app/design/adminhtml/default/default/layout/tnw_accountingseed.xml',
        'app/etc/modules/TNW_Accountingseed.xml',

        'app/code/community/TNW/Salesforce',
        'app/design/adminhtml/default/default/layout/salesforce.xml',
        'app/design/adminhtml/default/default/template/salesforce',
        'app/etc/modules/TNW_Salesforce.xml',
        'app/locale/en_US/TNW_Salesforce.csv',
        'app/locale/en_US/template/email/tnw_salesforce',
        'js/tnw-salesforce',
        'lib/Salesforce',
        'powersync/import.php',
        'powersync/sf-sync.php',
        'powersync/'
    );
    protected $_moduleSqls = array(

        // drop our tables
        'drop_table' => array(
            'tnw_accountingseed/sales_item_communication' => 1,
            'tnw_quote/mapping_quote_status' => 1,
            'tnw_salesforce/account_matching' => 1,
            'tnw_salesforce/entity_cache' => 1,
            'tnw_salesforce/import' => 1,
            'tnw_salesforce/log' => 1,
            'tnw_salesforce/mapping' => 1,
            'tnw_salesforce/order_creditmemo_status' => 1,
            'tnw_salesforce/order_status' => 1,
            'tnw_salesforce/queue' => 1,
            'tnw_salesforce/queue_storage' => 1
        ),
        // remove table columns
        'alter_table' => array(
            'sales_flat_creditmemo' => array(
                'salesforce_id',
                'sf_insync',
                'salesforce_account_payable_id',
                'salesforce_cash_disbursement_id'
            ),

            'catalogrule' => array(
                'sf_insync',
                'salesforce_id'
            ),

            'sales_flat_creditmemo_comment' => 'salesforce_id',
            'sales_flat_creditmemo_item' => 'salesforce_inbound_inventory_movement',

            'sales_flat_invoice' => array(
                'salesforce_id',
                'sf_insync',
                'ac_salesforce_id'),

            'sales_flat_invoice_comment' => array(
                'salesforce_id',
                'ac_salesforce_id'),

            'sales_flat_invoice_item' => array(
                'salesforce_id',
                'ac_salesforce_id'),

            'sales_flat_order' => array(
                'salesforce_id',
                'sf_insync',
                'contact_salesforce_id',
                'account_salesforce_id',
                'opportunity_id',
                'owner_salesforce_id'),

            'sales_flat_order_item' => array(
                'salesforce_id',
                'opportunity_id'),

            'sales_flat_quote' => array(
                'salesforce_id',
                'sf_insync',
                'sf_sync_force',
                'contact_salesforce_id',
                'account_salesforce_id'),

            'sales_flat_quote_item' => 'salesforce_id',

            'sales_flat_shipment' => array(
                'sf_insync',
                'salesforce_id',
                'ac_salesforce_id'),

            'sales_flat_shipment_comment' => array(
                'salesforce_id',
                'ac_salesforce_id'),

            'sales_flat_shipment_item' => array(
                'salesforce_id',
                'ac_salesforce_id'),

            'salesrule' => array(
                'sf_insync',
                'salesforce_id'),

            'wishlist' => array(
                'sf_insync',
                'salesforce_id'),

            'core_website' => array(
                'salesforce_id',
                'pricebook_id'),

            'quoteadv_customer' => array(
                'salesforce_quote_id',
                'salesforce_new_quote',
                'salesforce_id',
                'sf_insync',
                'salesforce_owner_id',
                'salesforce_initial_owner_id'
            ),
            'sales_flat_order_payment' => array(
                'salesforce_id'
            ),
            'sales_flat_order_status_history' => array(
                'salesforce_id',
                'opportunity_id'
            ),
            'quoteadv_crmaddon_messages' => array(
                'salesforce_id'
            )
        ),
        'delete' => array(
            'eav_attribute' => "DELETE FROM %s WHERE attribute_code IN ('salesforce_account_id','salesforce_account_owner_id','salesforce_campaign_id','salesforce_contact_owner_id','salesforce_disable_sync','salesforce_id','salesforce_is_person','salesforce_lead_id','salesforce_lead_owner_id','salesforce_pricebook_id', 'sf_insync')",
            'core_resource' => "DELETE FROM %s WHERE code IN ('tnw_accountingseed_setup', 'tnw_salesforce_setup', 'tnw_quote_setup')",
            'core_cache_option' => "DELETE FROM %s WHERE code IN ('tnw_salesforce')",
            'core_config_data' => "DELETE FROM %s WHERE path LIKE 'salesforce/%%' OR path LIKE 'salesforce_customer/%%' OR path LIKE 'salesforce_product/%%' OR path LIKE 'salesforce_order/%%' OR path LIKE 'salesforce_contactus/%%' OR path LIKE 'salesforce_promotion/%%' OR path LIKE 'salesforce_invoice/%%' OR  path LIKE 'salesforce_shipment/%%' OR path LIKE 'salesforce_creditmemo/%%' "

        )
    );

    /**
     * @param $message
     */
    protected function _log($message)
    {
        echo $message . "\n";
        Mage::log($message, Zend_Log::DEBUG, self::LOG_NAME);
    }

    /**
     * @return bool
     */
    protected function _checkArgs()
    {
        $hasArgs = false;
        foreach (array('files', 'dbChanges', 'showQueries') as $arg) {
            $hasArgs |= (bool)$this->getArg($arg);
        }
        return $hasArgs;
    }

    /**
     *
     */
    protected function _checkshowQueries()
    {
        if ($this->getArg('showQueries')) {
            $this->_log('The "showQueries" flag defined! No changes will be applied.');
            $this->_log('Necessary operations will be just printed. You can make necessary changes manually.');
        }
    }

    /**
     * @return bool
     */
    protected function _removeFiles()
    {
        if (!$this->getArg('files')) {
            return false;
        }

        $this->_log('Module files removing:');
        $this->_log('---------START---------');

        $io = new Varien_Io_File();
        foreach ($this->_moduleFiles as $file) {

            if (empty($file)) {
                continue;
            }

            $magentoDir = Mage::getBaseDir();
            $file = $magentoDir . DS . $file;

            if (!file_exists($file)) {
                $this->_log("Skip file: $file . Not exists.");
                continue;
            }

            $this->_log("Remove: $file");
            if (!$this->getArg('showQueries')) {

                try {

                    if (is_dir($file) && !is_link($file)) {
                        $result = $io->rmdir($file, true);
                    } else {
                        $result = $io->rm($file);
                    }
                    if (!$result) {
                        throw new Exception('Cannot remove the file');
                    }
                    $this->_log(">>> Done ");

                } catch (Exception $e) {
                    $this->_success = false;
                    $this->_log("File operation error: " . $e->getMessage());
                }
            }
        }

        $this->_log('---------END---------');

        return true;
    }

    /**
     * @return bool
     */
    protected function _removeDbChanges()
    {
        if (!$this->getArg('dbChanges')) {
            return false;
        }

        $this->_log('Module Database changes removing:');
        $this->_log('---------START---------');

        /** @var Mage_Core_Model_Resource $resource */
        $resource = Mage::getSingleton('core/resource');
        /**
         * Retrieve the write connection
         */
        $writeConnection = $resource->getConnection('core_write');
        foreach ($this->_moduleSqls as $type => $data) {

            try {

                if (!is_array($data)) {
                    throw new Exception("Query is empty, type: '$type', data: '" . var_export($data, true) . "'");
                }

                foreach ($data as $key => $value) {
                    try {
                        $sql = '';

                        try {
                            $tableName = $resource->getTableName($key);
                        } catch (Exception $e) {
                            $this->_log("Skip: Table not exists: " . $key);

                            continue;
                        }

                        switch ($type) {
                            case 'drop_table':
                                $sql = sprintf('DROP TABLE IF EXISTS %s ', $tableName);
                                break;
                            case 'alter_table':
                                if (!is_array($value)) {
                                    $value = array($value);
                                }
                                foreach ($value as $column) {
                                    $sql .= sprintf('ALTER TABLE %s DROP %s ;', $tableName, $column);
                                }

                                break;
                            case 'delete':
                                $sql = sprintf($value, $tableName);
                                break;
                            default:
                                throw new Exception('Incompatible sql Query type: "' . $type . '", data: ' . var_export($data, true));
                        }
                        $this->_log($sql);

                        if (!$this->getArg('showQueries')) {
                            $writeConnection->query($sql);
                            $this->_log(">>> Done ");
                        }
                    } catch (Exception $e) {
                        if ($e->getCode() == '42000') {
                            $this->_log("Skip: Changes applied already.");
                        } else {
                            $this->_success = false;
                            $this->_log(" !!! SQL Error: " . $e->getMessage());
                        }
                    }
                }
            } catch (Exception $e) {
                $this->_success = false;
                $this->_log(" !!! Error: " . $e->getMessage());
            }
        }

        $this->_log('---------END---------');

        return true;
    }

    /**
     * Run script
     *
     */
    public function run()
    {
        try {
            if ($this->_checkArgs()) {
                $this->_checkshowQueries();
                $this->_removeFiles();
                $this->_removeDbChanges();
            } else {
                echo $this->usageHelp();
            }
        } catch (Exception $e) {
            $this->_success = false;
            $this->_log(" !!! Error: " . $e->getMessage());
        }

        if (!$this->_success) {
            $this->_log("Some problems occur through the uninstall process, take a look messages above or log file: " . self::LOG_NAME);
        } else {
            $this->_log('SUCCESS!');
        }
    }

    /**
     * Retrieve Usage Help Message
     *
     */
    public function usageHelp()
    {
        return <<<USAGE
Usage:  php -f sf-remove.php -- [options]
        php -f sf-remove.php -- files dbChanges
  files            remove TNW_Salesforce module files 
  dbChanges        remove TNW_Salesforce module database changes
  showQueries         don't remove anything, just print queries and filenames should be removed
  help             This help
USAGE;
    }
}

$shell = new Mage_Shell_Log();
$shell->run();
